from pgzanimation import Slide
from pgzanimation import AnimActor, AnimText
import pygame, sys

WIDTH = 1280
HEIGHT = 720
FRAMES = 3500
TITLE = "PgzAnimation"

# Save options
# Continue to save frames when paused?
SAVE_PAUSED = True
# enable to show frame number on screen (not included in save)
SHOW_FRAME = True
SHOW_MOUSE = True
# Exit after last frame?
QUIT_END = False

# Is save enabled (otherwise just display animation)
save = True
save_files = "/home/stewart/animations/07-use/animation-{0:05d}.png"
# frame is the animation frame number
# when pause then save_frame continues to count to extend video length
frame=0
save_frame=0
pause = False

# For transparent background use 0,255,0 - then remove in video editor
background_color = (0,255,0)
background_image = "background02.png"

text_entries = [
    "from pgzanimation import AnimActor",
    "save = False",
    "save_files = \"/home/stewart/animations/03a-youtube/animation-{0:05d}.png\"",
    "background_color = (0,255,0)",
    "background_image = \"\""
    ]

slides = [
    Slide ((WIDTH,HEIGHT), "How to use")
    ]

shapes = {
    'code01': AnimActor ("ytani-code-01.png", (640, 400), hide=True),
    'code02': AnimActor ("ytani-code-02.png", (640, 400), hide=True),
    'code03': AnimActor ("ytani-code-03.png", (640, 400), hide=True),
    'convert1': AnimText ("ffmpeg -framerate 25 -i animation-%05d.png -c:v libx264 -profile:v ", (100, 300), anchor=("left", "center"), fontsize=25, fontname="droidsansmono", mode="typewriter", hide=False),
    'convert2': AnimText ("         high -crf 20 -pix_fmt yuv420p output.mp4 ", (100, 360), anchor=("left", "center"), fontsize=25, fontname="droidsansmono", mode="typewriter", hide=False)
    }

# Add AnimText to the shapes
for i in range (0, 5):
    shapes[i] = AnimText (text_entries[i], (100, 300 + (i * 60)), anchor=("left", "center"), fontsize=25, fontname="droidsansmono", mode="typewriter", hide=False)

# Add any pauses to this
# Pauses can also result from certain animations (eg. after each bullet)
# Add any pauses to this
pause_frames = []

# key frames (can just use number if preferred, but this allows labels)
kf = {
    'start-text': 100,
    'end-text': 1400,
    'code01': 1500,
    'code02': 1800,
    'code03': 2100,
    'convert': 2600
    }


slides[0].show_slide(0, FRAMES)

def draw():
    screen.clear()
    screen.fill(background_color)
    if (background_image != ""):
        screen.blit(background_image, (0,0))

    for this_shape in shapes.values():
        this_shape.draw()
    for this_slide in slides:
        this_slide.draw()
        
    # Save animation frame
    if (save == True and ( pause == False or SAVE_PAUSED == True)):
        pygame.image.save(screen.surface, save_files.format(save_frame))
    # Anything below here is displayed on screen, but not included in saves
    if SHOW_FRAME:
        # frame is incremented at end - so display number of previous frame
        screen.draw.text(str(frame-1), (20,20), color=(250, 50, 50), fontsize=60)
    if SHOW_MOUSE:
        screen.draw.text("Mouse "+str(pygame.mouse.get_pos()), (20, 60), color=(50, 50, 250), fontsize=60)


def update():
    controls()
    animate()

# Place your animations here
def animate():
    # Add animations here
    start_typewriter = kf['start-text']
    for i in range (0, 5):
        shapes[i].typewriter_tween (start_typewriter, start_typewriter+150, frame)
        start_typewriter += 250

    if frame == kf['end-text']:
        for i in range (0, 5):
            shapes[i].hide = True

    if frame == kf['code01']:
        shapes['code01'].hide=False
    if frame == kf['code01']+250:
        shapes['code01'].hide=True
    if frame == kf['code02']:
        shapes['code02'].hide=False
    if frame == kf['code02']+250:
        shapes['code02'].hide=True
    if frame == kf['code03']:
        shapes['code03'].hide=False
    if frame == kf['code03']+250:
        shapes['code03'].hide=True

    if frame == kf['convert'] :
        slides[0].title = "How to - convert to MP4"

    shapes['convert1'].typewriter_tween (kf['convert'], kf['convert']+250, frame)
    shapes['convert2'].typewriter_tween (kf['convert']+200, kf['convert']+450, frame)



def controls():
    global frame, save_frame, pause
    # Stop updating once end reached
    if (frame >= FRAMES):
        if (QUIT_END):
            pygame.quit()
            sys.exit()
        return
        # Are we currently paused
    if (pause == True):
        if (SAVE_PAUSED):
            save_frame += 1
        return

    # slides need to check for pause
    for this_slide in slides:
        if (this_slide.update(frame) == True): pause=True

    frame += 1
    save_frame += 1
    print ("Frame is {}".format(frame))

# Handle keys - when released
def on_key_up (key, mod):
    global pause
    # quit on q / esc
    if (key == keys.ESCAPE or key == keys.Q):
        pygame.quit()
        sys.exit()
    # Space / p to pause / unpause
    if (key == keys.SPACE or key == keys.P):
        pause = not pause

# Handle mouse up (to remove pause)
def on_mouse_up (pos, button):
    global pause
    pause = False